@@ -33,6 +33,7 @@
 */
 
 #include "pl-incl.h"
+#include "pl-comp.h"
 #include "pl-dict.h"
 #include "pl-rsort.h"
 
@@ -141,12 +142,6 @@ get_create_dict_ex(term_t t, term_t dt ARG_LD)
 }
 
 
-static inline int
-is_key(word w)
-{ return isAtom(w) || isTaggedInt(w);
-}
-
-
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 dict_lookup_ptr() returns a pointer to the value for a given key
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -199,25 +194,25 @@ dict_ordered(Word data, int count, int ex ARG_LD)
   if ( count > 0 )
   { data++;			/* skip to key */
     deRef2(data, n1);
-    if ( !is_key(*n1) )
+    if ( !is_dict_key(*n1) )
       return -1;
-  }
 
-  for(; count > 1; count--, data += 2, n1=n2)
-  { deRef2(data+2, n2);
-    if ( !is_key(*n2) )
-      return -1;
-    if ( *n1 < *n2 )
-      continue;
-    if ( *n1 > *n2 )
-      ordered = FALSE;
-    if ( *n1 == *n2 )
-    { if ( ex )
-      { term_t t = PL_new_term_ref();
-	*valTermRef(t) = linkVal(n1);
-	PL_error(NULL, 0, NULL, ERR_DUPLICATE_KEY, t);
+    for(; count > 1; count--, data += 2, n1=n2)
+    { deRef2(data+2, n2);
+      if ( !is_dict_key(*n2) )
+	return -1;
+      if ( *n1 < *n2 )
+	continue;
+      if ( *n1 > *n2 )
+	ordered = FALSE;
+      if ( *n1 == *n2 )
+      { if ( ex )
+	{ term_t t = PL_new_term_ref();
+	  *valTermRef(t) = linkVal(n1);
+	  PL_error(NULL, 0, NULL, ERR_DUPLICATE_KEY, t);
+	}
+	return -2;
       }
-      return -2;
     }
   }
 
@@ -591,7 +586,7 @@ get_name_ex(term_t t, Word np ARG_LD)
 { Word p = valTermRef(t);
 
   deRef(p);
-  if ( is_key(*p) )
+  if ( is_dict_key(*p) )
   { *np = *p;
     return TRUE;
   }
@@ -616,7 +611,7 @@ get_name_value(Word p, Word name, Word value, Word mark, int flags ARG_LD)
     { Word np, vp;
 
       deRef2(&f->arguments[0], np);
-      if ( is_key(*np) )
+      if ( is_dict_key(*np) )
       { *name = *np;
 	deRef2(&f->arguments[1], vp);
 	*value = linkVal(vp);
@@ -861,7 +856,7 @@ PL_for_dict(term_t dict,
 
 This predicate is called from pl-wic.c after   reloading a clause from a
 .qlf file or state if pl-wic.c  detected   a  dict  inside the clause. It
-identifies the code ranges that constitude the  k-v pairs in the dict and
+identifies the code ranges that constitute the  k-v pairs in the dict and
 re-orders them according to the new atom-handle ordering.
 
 There    is    a    complicating     factor    with    B_FIRSTVAR/B_VAR,
@@ -1161,7 +1156,7 @@ pl_get_dict(term_t PL__t0, int PL__ac, int ex, control_t PL__ctx)
 	return FALSE;
 
       deRef(np);
-      if ( is_key(*np) )
+      if ( is_dict_key(*np) )
       { Word vp;
 
 	if ( (vp=dict_lookup_ptr(dict, *np PASS_LD)) )
@@ -1654,6 +1649,29 @@ retry:
     }
   }
 
+  return FALSE;
+}
+
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Part of FLI
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+int
+PL_get_dict_key(atom_t key, term_t dict, term_t value)
+{ GET_LD
+  word d;
+  Word vp;
+
+  if ( !is_dict_key(key) )
+    return -1;
+  if ( !get_dict_ex(dict, &d, FALSE PASS_LD) )
+    return FALSE;
+  if ( (vp=dict_lookup_ptr(d, key PASS_LD)) )
+  { *valTermRef(value) = linkVal(vp);
+    return TRUE;
+  }
+
   return FALSE;
 }
 
