@@ -115,14 +115,12 @@ static const PL_extension foreigns[] = {
 
   FRG("abolish",		1, pl_abolish1,		     META|ISO),
   FRG("abolish",		2, pl_abolish,		     META),
-  FRG("nth_clause",		3, pl_nth_clause,       NDET|META|CREF),
 #ifdef O_MAINTENANCE
   FRG("$list_generations",	1, pl_list_generations,	     META),
   FRG("$check_procedure",	1, pl_check_procedure,	     META),
 #endif
 
   FRG("$c_current_predicate",	2, pl_current_predicate,  NDET|META),
-  FRG("current_predicate",	1, pl_current_predicate1, NDET|META|ISO),
   FRG("$require",		1, pl_require,		     META),
 
   FRG("repeat",			0, pl_repeat,		 NDET|ISO),
@@ -204,6 +202,21 @@ static const PL_extension foreigns[] = {
 };
 
 
+#define SIGNATURE_SEED (0x1a3be34a)
+
+static unsigned int
+predicate_signature(const Definition def)
+{ char str[256];
+
+  Ssprintf(str, "%s/%d/%d",
+	   stringAtom(def->functor->name),
+	   (int)def->functor->arity,
+	   def->flags);
+
+  return MurmurHashAligned2(str, strlen(str), SIGNATURE_SEED);
+}
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 The extensions chain is used   to allow calling PL_register_extensions()
 *before* PL_initialise() to get foreign   extensions in embedded systems
@@ -324,6 +337,9 @@ registerBuiltins(const PL_extension *f)
 
       def->impl.foreign.function = f->function;
       createForeignSupervisor(def, f->function);
+
+      if ( !extensions_loaded )
+	GD->foreign.signature ^= predicate_signature(def);
     } else
     { assert(0);
     }
