@@ -114,48 +114,6 @@ COMMON(size_t)		atom_space(void);
 /* pl-bag.c */
 COMMON(void)		markAtomsFindall(PL_local_data_t *ld);
 
-/* pl-comp.c */
-COMMON(void)		initWamTable(void);
-COMMON(void)		freeVarDefs(PL_local_data_t *ld);
-COMMON(int)		get_head_and_body_clause(term_t clause,
-					 term_t head, term_t body,
-					 Module *m ARG_LD);
-COMMON(Procedure)	lookupBodyProcedure(functor_t functor, Module tm);
-COMMON(int)		compileClause(Clause *cp, Word head, Word body,
-				      Procedure proc, Module module,
-				      term_t warnings ARG_LD);
-COMMON(Clause)		assert_term(term_t term, ClauseRef where, atom_t owner,
-				    SourceLoc loc ARG_LD);
-COMMON(void)		forAtomsInClause(Clause clause, void (func)(atom_t a));
-COMMON(Code)		stepDynPC(Code PC, const code_info *ci);
-COMMON(bool)		decompileHead(Clause clause, term_t head);
-COMMON(Code)		skipArgs(Code PC, int skip);
-COMMON(int)		argKey(Code PC, int skip, word *key);
-COMMON(int)		arg1Key(Code PC, word *key);
-COMMON(bool)		decompile(Clause clause, term_t term, term_t bindings);
-COMMON(word)		pl_nth_clause(term_t p, term_t n, term_t ref,
-				      control_t h);
-COMMON(void)		wamListClause(Clause clause);
-COMMON(Code)		wamListInstruction(IOSTREAM *out, Code relto, Code bp);
-COMMON(int)		unify_definition(Module ctx, term_t head, Definition def,
-					 term_t thehead, int flags);
-COMMON(code)		replacedBreak(Code PC);
-COMMON(code)		replacedBreakUnlocked(Code PC);
-COMMON(int)		clearBreakPointsClause(Clause clause) WUNUSED;
-COMMON(int)		unify_functor(term_t t, functor_t fd, int how);
-COMMON(void)		vm_list(Code code);
-COMMON(Module)		clauseBodyContext(const Clause cl);
-
-/* pl-dump.c */
-COMMON(word)		saveProgram(term_t new);
-COMMON(word)		pl_save_program(term_t new, term_t args);
-COMMON(word)		pl_save(term_t file, term_t restore);
-COMMON(word)		pl_restore(term_t file);
-COMMON(word)		parseSaveProgramOptions(term_t args,
-			int *local, int *global, int *trail, int *argument,
-			char **goal, char **toplevel, char **init_file,
-			bool *tty, bool *standalone);
-
 /* pl-index.c */
 COMMON(word)		getIndexOfTerm(term_t t);
 COMMON(ClauseRef)	firstClause(Word argv, LocalFrame fr, Definition def,
@@ -172,6 +130,7 @@ COMMON(void)		unallocClauseIndexTable(ClauseIndex ci);
 COMMON(void)		deleteActiveClauseFromIndexes(Definition def, Clause cl);
 COMMON(bool)		unify_index_pattern(Procedure proc, term_t value);
 COMMON(void)		deleteIndexes(ClauseList cl, int isnew);
+COMMON(void)		deleteIndexesDefinition(Definition def);
 COMMON(int)		checkClauseIndexSizes(Definition def, int nindexable);
 COMMON(void)		checkClauseIndexes(Definition def);
 COMMON(void)		listIndexGenerations(Definition def, gen_t gen);
@@ -355,7 +314,9 @@ COMMON(word)		pl_import(term_t pred);
 #ifdef O_PROLOG_HOOK
 COMMON(word)		pl_set_prolog_hook(term_t module, term_t old, term_t new);
 #endif
-
+COMMON(ModuleEnum)	newModuleEnum(int flags);
+COMMON(Module)		advanceModuleEnum(ModuleEnum en);
+COMMON(void)		freeModuleEnum(ModuleEnum en);
 
 /* pl-op.c */
 COMMON(int)		currentOperator(Module m, atom_t name, int kind,
@@ -439,6 +400,7 @@ COMMON(int)		setDoubleQuotes(atom_t a, unsigned int *flagp);
 COMMON(int)		setBackQuotes(atom_t a, unsigned int *flagp);
 COMMON(int)		setRationalSyntax(atom_t a, unsigned int *flagp);
 COMMON(void)		initPrologFlags(void);
+COMMON(void)		setABIVersionPrologFlag(void);
 COMMON(void)		cleanupPrologFlags(void);
 
 /* pl-pro.c */
@@ -477,7 +439,6 @@ COMMON(int)		get_procedure(term_t descr, Procedure *proc,
 COMMON(int)		checkModifySystemProc(functor_t f);
 COMMON(int)		overruleImportedProcedure(Procedure proc, Module target);
 COMMON(word)		pl_current_predicate(term_t name, term_t functor, control_t h);
-COMMON(foreign_t)	pl_current_predicate1(term_t spec, control_t ctx);
 COMMON(void)		clear_meta_declaration(Definition def);
 COMMON(void)		setMetapredicateMask(Definition def, arg_info *args);
 COMMON(int)		isTransparentMetamask(Definition def, arg_info *args);
@@ -528,6 +489,7 @@ COMMON(int)		startConsult(SourceFile f);
 COMMON(int)		endConsult(SourceFile f);
 COMMON(size_t)		highSourceFileIndex(void);
 COMMON(SourceFile)	lookupSourceFile(atom_t name, int create);
+COMMON(int)		releaseSourceFileNo(int index);
 COMMON(SourceFile)	indexToSourceFile(int index);
 COMMON(void)		cleanupSourceFiles(void);
 COMMON(void)		unlinkSourceFileModule(SourceFile sf, Module m);
@@ -544,6 +506,18 @@ COMMON(int)		exportProcedureSource(SourceFile sf, Module module,
 					      Procedure proc);
 COMMON(void)		registerReloadModule(SourceFile sf, Module module);
 
+#ifdef O_DEBUG
+COMMON(void)		acquireSourceFile_d(SourceFile f,
+					    const char *file, unsigned int line);
+COMMON(int)		releaseSourceFile_d(SourceFile f,
+					    const char *file, unsigned int line);
+#define acquireSourceFile(f) acquireSourceFile_d(f, __FILE__, __LINE__)
+#define releaseSourceFile(f) releaseSourceFile_d(f, __FILE__, __LINE__)
+#else
+COMMON(void)		acquireSourceFile(SourceFile sf);
+COMMON(int)		releaseSourceFile(SourceFile f);
+#endif
+
 /* pl-read.c */
 COMMON(void)		resetRead(void);
 COMMON(int)		f_is_prolog_var_start(wint_t c);
