@@ -38,6 +38,7 @@
 
 #define GLOBAL SO_LOCAL			/* allocate global variables here */
 #include "pl-incl.h"
+#include "pl-comp.h"
 #include "pl-arith.h"
 #include "os/pl-cstack.h"
 #include "pl-dbref.h"
@@ -141,6 +142,7 @@ setupProlog(void)
 #ifdef O_LOCALE
   initLocale();
 #endif
+  setABIVersionPrologFlag();
   GD->io_initialised = TRUE;
   GD->clauses.cgc_space_factor  = 8;
   GD->clauses.cgc_stack_factor  = 0.03;
@@ -1592,16 +1594,8 @@ of work to do.
 
 void
 freePrologLocalData(PL_local_data_t *ld)
-{ int i;
-
-  discardBuffer(&ld->fli._discardable_buffer);
-
-  for(i=0; i<BUFFER_RING_SIZE; i++)
-  { discardBuffer(&ld->fli._buffer_ring[i]);
-    initBuffer(&ld->fli._buffer_ring[i]);	/* Used by debug-print */
-						/* on shutdown */
-  }
-
+{ discardBuffer(&ld->fli._discardable_buffer);
+  discardStringStack(&ld->fli.string_buffers);
   freeVarDefs(ld);
 
 #ifdef O_GVAR
