@@ -36,6 +36,7 @@
 
 /*#define O_DEBUG 1*/
 #include "pl-incl.h"
+#include "pl-comp.h"
 #include "pl-arith.h"
 #include "pl-inline.h"
 #include "pl-dbref.h"
@@ -278,11 +279,14 @@ updateAlerted(PL_local_data_t *ld)
 #ifdef O_DEBUGGER
   if ( ld->_debugstatus.debugging )		mask |= ALERT_DEBUG;
 #endif
+  if ( ld->fli.string_buffers.top )		mask |= ALERT_BUFFER;
 
   ld->alerted = mask;
 
   if ( (mask&ALERT_DEBUG) || ld->prolog_flag.occurs_check != OCCURS_CHECK_FALSE )
     ld->slow_unify = TRUE;		/* see VMI B_UNIFY_VAR */
+  else
+    ld->slow_unify = FALSE;
 }
 
 
@@ -1385,20 +1389,18 @@ static Definition
 localDefinition(Definition def ARG_LD)
 { unsigned int tid = LD->thread.info->pl_tid;
   size_t idx = MSB(tid);
-  LocalDefinitions v = def->impl.local;
+  LocalDefinitions v = def->impl.local.local;
 
   if ( !v->blocks[idx] )
-  { if ( !v->blocks[idx] )
-    { size_t bs = (size_t)1<<idx;
-      Definition *newblock;
+  { size_t bs = (size_t)1<<idx;
+    Definition *newblock;
 
-      if ( !(newblock=PL_malloc_uncollectable(bs*sizeof(Definition))) )
-	outOfCore();
+    if ( !(newblock=PL_malloc_uncollectable(bs*sizeof(Definition))) )
+      outOfCore();
 
-      memset(newblock, 0, bs*sizeof(Definition));
-      if ( !COMPARE_AND_SWAP_PTR(&v->blocks[idx], NULL, newblock-bs) )
-	PL_free(newblock);
-    }
+    memset(newblock, 0, bs*sizeof(Definition));
+    if ( !COMPARE_AND_SWAP_PTR(&v->blocks[idx], NULL, newblock-bs) )
+      PL_free(newblock);
   }
 
   if ( !v->blocks[idx][tid] )
@@ -1410,7 +1412,7 @@ localDefinition(Definition def ARG_LD)
 void
 destroyLocalDefinition(Definition def, unsigned int tid)
 { size_t idx = MSB(tid);
-  LocalDefinitions v = def->impl.local;
+  LocalDefinitions v = def->impl.local.local;
   Definition local;
 
   local = v->blocks[idx][tid];
@@ -1424,7 +1426,8 @@ getProcDefinition__LD(Definition def ARG_LD)
 {
 #ifdef O_PLMT
   if ( true(def, P_THREAD_LOCAL) )
-  { return localDefinition(def PASS_LD);
+  { MEMORY_ACQUIRE();
+    return localDefinition(def PASS_LD);
   }
 #endif
 
@@ -1435,7 +1438,7 @@ getProcDefinition__LD(Definition def ARG_LD)
 Definition
 getProcDefinitionForThread(Definition def, unsigned int tid)
 { size_t idx = MSB(tid);
-  LocalDefinitions v = def->impl.local;
+  LocalDefinitions v = def->impl.local.local;
 
   if ( !v->blocks[idx] )
     return NULL;
@@ -2296,7 +2299,9 @@ discardChoicesAfter(LocalFrame fr, enum finished reason ARG_LD)
       for(fr2 = me->frame;
 	  fr2 > delto;
 	  fr2 = fr2->parent)
-      { assert(fr2->clause || true(fr2->predicate, P_FOREIGN));
+      { assert(onStack(local, me));
+	assert(onStack(local, fr2));
+	assert(fr2->clause || true(fr2->predicate, P_FOREIGN));
 
 	if ( true(fr2, FR_WATCHED) )
 	{ char *lSave = (char*)lBase;
@@ -2306,17 +2311,16 @@ discardChoicesAfter(LocalFrame fr, enum finished reason ARG_LD)
 	    Undo(me->mark);
 	    DiscardMark(me->mark);
 	  }
-	  BFR = me->parent;
+	  BFR = me;
 	  frameFinished(fr2, reason PASS_LD);
+	  BFR = BFR->parent;
 	  if ( lSave != (char*)lBase )	/* shifted */
 	  { intptr_t offset = (char*)lBase - lSave;
 
 	    me  = addPointer(me, offset);
-	    me->parent = BFR;		/* not updated because BFR=me->parent */
 	    fr  = addPointer(fr, offset);
 	    fr2 = addPointer(fr2, offset);
 	    delto = addPointer(delto, offset);
-	    fr2->parent = addPointer(fr2->parent, offset);
 	  }
 #if 0					/* What to do if we have multiple */
 	  if ( exception_term )		/* handlers and multiple exceptions? */
@@ -2749,12 +2753,6 @@ PL_yielded(qid_t qid)
 	  qf->registers.fr = NULL; \
 	}
 
-#ifndef ASM_NOP
-int _PL_nop_counter;
-
-#define ASM_NOP _PL_nop_counter++
-#endif
-
 typedef enum
 { uread = 0,				/* Unification in read-mode */
   uwrite				/* Unification in write mode */
@@ -2829,10 +2827,13 @@ pl-comp.c
 				     DbgPrintInstruction(FR, PC); \
 				     goto *(void *)((intptr_t)(*PC++)); \
 				   } while(0)
-#ifndef ASM_NOP
-#define ASM_NOP asm("nop")
-#endif
-#define SEPERATE_VMI ASM_NOP
+
+/* This macro must ensure that two identical VMI instructions do not get
+ * merged onto the same address by the compiler, causing decompilation
+ * which translates the addresses back into the VMI number to fail.
+ * initWamTable() verfies this does not happen.
+ */
+#define SEPARATE_VMI { static volatile int nop = 0; (void)nop; }
 
 #else /* VMCODE_IS_ADDRESS */
 
@@ -2855,7 +2856,7 @@ code thiscode;
 				  END_PROF(); \
                                   goto next_instruction; \
 				}
-#define SEPERATE_VMI		(void)0
+#define SEPARATE_VMI		(void)0
 
 #endif /* VMCODE_IS_ADDRESS */
 
@@ -3208,6 +3209,11 @@ next_choice:
   last_choice = ch->type;
 #endif
 
+  if ( (LD->alerted & ALERT_BUFFER) )
+  { LD->alerted &= ~ALERT_BUFFER;
+    release_string_buffers_from_frame(FR PASS_LD);
+  }
+
   switch(ch->type)
   { case CHP_JUMP:
       DEBUG(MSG_BACKTRACK,
@@ -3291,6 +3297,7 @@ next_choice:
 	  { int action;
 
 	    SAVE_REGISTERS(qid);
+	    clearLocalVariablesFrame(FR);
 	    action = tracePort(fr, BFR, REDO_PORT, NULL PASS_LD);
 	    LOAD_REGISTERS(qid);
 	    ch = BFR;			/* can be shifted */
